/*
 * $Header: /home/cvs/jakarta-slide/src/contrib/webdavgui/src/java/org/apache/webdav/ui/filechooser/dir/SPDirectoryService.java,v 1.2 2001/07/17 04:00:47 msmith Exp $
 * $Revision: 1.2 $
 * $Date: 2001/07/17 04:00:47 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 

package org.apache.webdav.ui.filechooser.dir;

import javax.swing.*;
import java.io.*;
import java.util.*;

import org.apache.webdav.lib.*;
import org.apache.webdav.ui.lib.methods.SPPropFindMethod;
import org.apache.webdav.ui.util.SPUtils;

import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.HttpClient;

/**
 * Title:        SPDirectoryService.java
 * Description:  Provides services which are related to directory queries to
 *               SPFileChooser, SPFile, SPFileView, SPFileSystemView,
 *               and SPFileFilter
 * Company:      SpeedLegal Holdings Inc.
 * @author       Jojada J. Tirtowidjojo
 * @version 1.0
 */

public class SPDirectoryService {

  /*
   * A file cache that holds SPFile as Key and
   * its associated SPResourceNode as Value
   */
  private Map fileCache = new HashMap();

  HttpClient client;
  SPFile root = null;
  SPFile homeDirectory;

  public SPDirectoryService(HttpClient client, String rootUrlPath)
                                                     throws Exception {
    this.client = client;
    SPWebFolder rootAsNode = new SPWebFolder(client, rootUrlPath);

    rootAsNode.init();
    rootAsNode.setParent(rootAsNode);
    if (System.getProperty("os.name").indexOf("Windows")>=0)  {
      	root = new SPFile("C:\\", rootAsNode.getName(), this);
    } else  {
    	root = new SPFile("/",rootAsNode.getName(), this);
    }

    this.fileCache.put(root, rootAsNode);
    homeDirectory = root; //by default
  }


  /*
   * Returns the children of a directory.
   */
  public File[] getChildren(File aDir)  {

    File[] arrFiles = new File[0];

    if (inCache(aDir)) {
//System.out.println("SPDirectorySErvice: getChildren of "+aDir.getAbsolutePath());
      SPResourceNode node = ((SPFile) aDir).getResourceNode();
      List children = node.getChildren();
      arrFiles = new File[children.size()];

      int k = 0;
      for (Iterator i=children.iterator(); i.hasNext();)  {
          SPResourceNode childNode = (SPResourceNode) i.next();
          File f = new SPFile(aDir, childNode.getName(), this);
          arrFiles[k++] = f;
          this.fileCache.put(f, childNode);
      }
    }//if (inCache(aDir))

    return arrFiles;
  }

  /*
   * Returns the root.
   */
  public SPFile getRoot()  {
    return root;
  }

  /*
   * Return the home directory.
   */
  public SPFile getHomeDirectory()  {
    return homeDirectory;
  }

  /*
   * Set the home directory
   * The homeFilePath argument must be an absolute path
   */
  public void setHomeDirectory(String homeFilePath, String homeUrlPath)
        throws Exception  {

    File f = new File(homeFilePath);
    if (! f.isAbsolute()) {
        throw new HttpException(
	    "Bad File Path: Requires an Absolute Home File Path");
    }

    homeDirectory = new SPFile(homeFilePath, this);

  }//setHomeDirectory()

  public void resetHomeDirectory()  {
    homeDirectory = root;
  }

  /*
   * Creates a new folder in the containing directory.
   */
  public File createNewFolder(File aContainingDir) {
    final String title = "New Folder Error";
    final String msg = "Service is not available yet.";
    JOptionPane.showMessageDialog(null, msg,
                title, JOptionPane.ERROR_MESSAGE);
    return null;
  }

  /*
   * Check whether the specified file exists in cache
   */
  boolean inCache(File aFile) {
    return fileCache.containsKey(aFile);
  }

 /*
  * Returns the associated SPResourceNode of aFile in fileCache
  */
  SPResourceNode getNodeFromCache(SPFile aFile)  {
    return (SPResourceNode) fileCache.get(aFile);
  }

  SPResourceNode getNodeFromServer(SPFile aFile) throws HttpException  {
    SPResourceNode theNode = (inCache(aFile))
				? getNodeFromCache(aFile)
				: expandCache(aFile);
    return theNode;
  }

  /*
   * To expand fileCache so that it may contain all files from root to
   * the specified f, file argument. A file must have its associated
   * resource node in the server in order to be put in fileCache.
   * An exeption is thrown when f, the file argument, is not under root.
   *
   * This method does not force the server to create a new resource node.
   *
   * @returns the associated SPResourceNode of f - the file argument -
   *          if f is available in fileCache or null otherwise
   */

  SPResourceNode expandCache(SPFile aFile) throws HttpException {
    SPResourceNode theNode = null;

    String rootPath = root.getAbsolutePath();
    String filePath = aFile.getAbsolutePath();
    if (filePath.indexOf(rootPath) < 0)  {
            throw (new HttpException("Bad File Path: "+filePath
	            +" NOT Under Root = "+rootPath));
    }

//System.out.println("SPDirectoryService: IN expandCache() for aFile = "+aFile.getAbsolutePath());

    /*
     * Find the immediate parent file which exists in fileCache
     */
    SPFile immediateParent = (SPFile) aFile.getParentFile();
    SPFile fileToBeChecked = aFile;

    while (! inCache(immediateParent))  {
	fileToBeChecked = immediateParent;
        immediateParent = (SPFile) immediateParent.getParentFile();
    }

//System.out.println("SPDirectoryService: expandCache(): getChildren of "
//+immediateParent.getAbsolutePath());

     //Refresh fileCache
    immediateParent.refreshContent();
    getChildren( new SPFile(immediateParent.getAbsolutePath(), this) );

    if (inCache(fileToBeChecked))  {
//System.out.println("SPDirectoryService: "+fileToBeChecked.getAbsolutePath()+" IN CACHE");
        if (fileToBeChecked.equals(aFile))  {
            theNode = getNodeFromCache(aFile);
	} else  {
	    theNode = expandCache(aFile);
	}
    }

    return theNode;

  } // expandCache()

  SPResourceNode createSPResourceNode(SPFile aFile)
        throws Exception {

      SPResourceNode theNode = getNodeFromServer(aFile);

      if (theNode == null)  { //try to create a new node in Server

	    if (! inCache(aFile.getParentFile()) )  {
	        throw new HttpException("Bad File Path: "+aFile.getParent());
	    }

	    SPWebFolder parentDir = (SPWebFolder)
		    getNodeFromCache((SPFile) aFile.getParentFile());

	    String parentUrlPath = parentDir.getResourceUrlPath();

	    if (parentDir.hasWriteAccess())   {
		SPWebFile newNode = new SPWebFile(client,
		    parentUrlPath+aFile.getName());
		int statusCode = newNode.putFileContent("");
		if ( statusCode == HttpStatus.SC_OK) {
		    parentDir.addWebFile(newNode);
		    this.fileCache.put(aFile, newNode);
		    theNode = (SPResourceNode) newNode;
		}
	    } else {
		throw new HttpException("Write Access Denied in "
		        +parentUrlPath+" ("+HttpStatus.SC_FORBIDDEN+")");
	    }
      }

      return theNode;
  }//createSPResourceNode()

}// End of SPDirectoryService class

//  SPResourceNode expandCache(SPFile aFile) throws WebdavException {
//    SPResourceNode theNode = null;
//
//    String rootPath = root.getAbsolutePath();
//    String filePath = aFile.getAbsolutePath();
//    if (filePath.indexOf(rootPath) < 0)  {
//            throw (new WebdavException("Bad File Path: "+filePath
//	            +" NOT Under Root = "+rootPath));
//    }
//
//System.out.println("SPDirectoryService: IN expandCache() for aFile = "+aFile.getAbsolutePath());
//
//    /*
//     * Find the immediate parent file which exists in fileCache
//     */
//    SPFile parentFile = (SPFile) aFile.getParentFile();
//    while (! inCache(parentFile))  {
//        parentFile = (SPFile) parentFile.getParentFile();
//    }
//
//System.out.println("SPDirectoryService: expandCache(): getChildren of "
//+parentFile.getAbsolutePath());
//
//     //Refresh fileCache
//    parentFile.refreshContent();
//    getChildren( new SPFile(parentFile.getAbsolutePath(), this) );
//
//    if (inCache(aFile))  {
//System.out.println("SPDirectoryService: "+aFile.getAbsoluteFile()+" IN CACHE");
//        theNode = getNodeFromCache(aFile);
//    } else {
//System.out.println("SPDirectoryService: "+aFile.getAbsoluteFile()+" NOT IN Cache");
//        if ( ! parentFile.equals(root)
//	        && (! parentFile.equals(aFile.getParentFile())) )  {
//	    theNode = expandCache(aFile);
//	}
//    }
//
//    return theNode;
//
//  } // expandCache()
