/*
 * $Header: /home/cvs/jakarta-slide/src/contrib/webdavgui/src/java/org/apache/webdav/ui/filechooser/dir/SPWebFile.java,v 1.3 2001/08/01 01:25:54 msmith Exp $
 * $Revision: 1.3 $
 * $Date: 2001/08/01 01:25:54 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 

package org.apache.webdav.ui.filechooser.dir;

import java.util.*;
import java.io.*;

import org.apache.webdav.lib.*;
import org.apache.webdav.ui.lib.methods.*;
import org.apache.webdav.lib.properties.*;
import org.apache.webdav.ui.util.SPUtils;

import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.HttpClient;


/**
 * Title:        SPWebFile.java
 * Description:  A Non-Collection Resource Node
 * Company:      SpeedLegal Holdings Inc.
 * @author       Jojada J. Tirtowidjojo
 * @version 1.0
 */

public class SPWebFile extends SPResourceNode {

  public SPWebFile(HttpClient client, String urlPath)   {
    this(client, urlPath, SPUtils.getFileNameOfUrl(urlPath));
  }

  public SPWebFile(HttpClient client, String urlPath, String name)  {
    this.client = client;
    this.resourceUrlPath = urlPath;
    this.name = name;
  }
/*
  public SPWebFile(HttpClient client, String urlPath, boolean readAccess,
            boolean writeAccess) {
    this(client, urlPath, SPUtils.getFileNameOfUrl(urlPath), readAccess, writeAccess);
  }

  public SPWebFile(HttpClient client, String urlPath, String name,
            boolean readAccess, boolean writeAccess) {
    this.client = client;
    this.resourceUrlPath = urlPath;
    this.name = name;
    this.readAccess = readAccess;
    this.writeAccess = writeAccess;
    accessPrivilegeChecked = true;
  }
*/
  public List getChildren()  {
    return null;
  }

  public boolean isWebFolder()  {
    return false;
  }

  public boolean isAcceptable(String aFilterDescription) {
    boolean isAcceptable = false;

    String extension = SPUtils.getExtension(name);
    int openingBrace = aFilterDescription.indexOf("(");
    int closingBrace = aFilterDescription.indexOf(")");

    String extList = aFilterDescription.substring(openingBrace+1, closingBrace);

    if (extension != null)  {
      if (extList.indexOf(extension) >= 0) {
        //listed
        isAcceptable = true;
      }
    }

    return isAcceptable;
  }

  public int getType()  {
    return SPResourceNode.NON_COLLECTION_TYPE;
  }

  public String getFileContentAsString(boolean writeLock) throws Exception  {
    return getFileContentAsString(writeLock, SPResourceNode.DEFAULT_TIMEOUT);
  }

  public String getFileContentAsString(boolean writeLock,
                                               long timeout) throws Exception  {
    String theString;

    if (writeLock)  {
      lock(timeout);
    }

    SPGetMethod getMethod = new SPGetMethod(this);
    getMethod.execute();

    theString = getMethod.getDataAsString();

    int statusCode = getMethod.getStatusCode();
    if (statusCode != HttpStatus.SC_OK) {
      final String title = "Import Failure";
      final String msg = "Server Response Status is NOT OK ("+statusCode+")\n"+
      "Resource Path = "+resourceUrlPath;
      SPUtils.showMessageDialog(title, msg);
    }

    return theString;
  } // getFileContentAsString()


  public InputStream getFileContentAsStream(boolean writeLock) throws Exception  {
    return getFileContentAsStream(writeLock, SPResourceNode.DEFAULT_TIMEOUT);
  }

  public InputStream getFileContentAsStream(boolean writeLock,
                                               long timeout) throws Exception  {
    InputStream theStream;

    if (writeLock)  {
      lock(timeout);
    }

    SPGetMethod getMethod = new SPGetMethod(client);
    getMethod.setPath(resourceUrlPath);
    getMethod.execute();

    theStream = getMethod.getData();

    int statusCode = getMethod.getStatusCode();

    if (statusCode != HttpStatus.SC_OK) {
      final String title = "Import Failure";
      final String msg = "Server Response Status is NOT OK ("+statusCode+")\n"+
      "Resource Path = "+resourceUrlPath;
      SPUtils.showMessageDialog(title, msg);
      theStream = null;
    }

    return theStream;
  } // getFileContentAsStream()

  public int putFileContent(String fileContent) throws Exception {

    if (! hasWriteAccess()) {
        String title = "Export Failure";
        String msg = "Write Access Denied ("+HttpStatus.SC_FORBIDDEN+")\n"+
              "Resource Path = "+resourceUrlPath;
        SPUtils.showMessageDialog(title, msg);
        return (HttpStatus.SC_FORBIDDEN);
    }

    SPPutMethod putMethod = new SPPutMethod(this);
    putMethod.sendData(fileContent);
    putMethod.execute();

    int statusCode = putMethod.getStatusCode();
    if (statusCode != HttpStatus.SC_OK) {
        String title = "Export Failure";
        String msg;

        if (statusCode == HttpStatus.SC_LOCKED) {
            msg = "Resource Path = "+resourceUrlPath+" is being LOCKED by another user.\n";
        } else {
            msg = "Server Response Status is NOT OK ("+statusCode+")\n"+
                         "Resource Path = "+resourceUrlPath;
        }
        SPUtils.showMessageDialog(title, msg);
    }

    return statusCode;
  }//putFileContent(String)

  public int putFileContent(File aFile) throws Exception {

    if (! hasWriteAccess()) {
        String title = "Export Failure";
        String msg = "Write Access Denied ("+HttpStatus.SC_FORBIDDEN+")\n"+
              "Resource Path = "+resourceUrlPath;
        SPUtils.showMessageDialog(title, msg);
        return (HttpStatus.SC_FORBIDDEN);
    }

    SPPutMethod putMethod = new SPPutMethod(this);
    putMethod.sendData(aFile);
    putMethod.execute();

    int statusCode = putMethod.getStatusCode();
    if (statusCode != HttpStatus.SC_OK) {
        String title = "Export Failure";
        String msg;

        if (statusCode == HttpStatus.SC_LOCKED) {
            msg = "Resource Path = "+resourceUrlPath
		+" is being LOCKED by another user.\n";
        } else {
            msg = "Server Response Status is NOT OK ("+statusCode+")\n"
		+"Resource Path = "+resourceUrlPath;
        }
        SPUtils.showMessageDialog(title, msg);
    }

    return statusCode;
  }//putFileContent(File)

  public void setProperties(Enumeration properties) {
	SPUtils.setProperty(this, properties);
  }

  public void refreshContent()  {
    //do nothing
  }

} //End of SPWebFile class
