/*
 * $Header: /home/cvs/jakarta-slide/src/share/org/apache/slide/common/CacheInfoToken.java,v 1.1 2001/05/14 06:35:16 remm Exp $
 * $Revision: 1.1 $
 * $Date: 2001/05/14 06:35:16 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 

package org.apache.slide.common;

/**
 * Caching strategy token class.
 * 
 * @author <a href="mailto:peter@kitten.net">Peter Hawkins</a>
 */
public final class CacheInfoToken {
    
    
    // ------------------------------------------------------------ Constructor
    
    
    /**
     * Constructor.
     * 
     * @param strategy describes the preferred strategy
     */
    public CacheInfoToken(String strategy) {
        this.strategy = strategy;
    }
    
    
    /**
     * Constructor.
     * 
     */
    public CacheInfoToken() {
    }
    
    
    // ----------------------------------------------------- Instance Variables
    
    
    /**
     * Strategy.
     */
    private String strategy;
    
    
    /**
     * Browsing depth. For example, it can be used to represent the depth of 
     * a PROPFIND.
     */
    private int searchDepth;

    
    /**
     * If true, the cache is used during a PROPFIND and then flushed.
     * This enables the use of a cache to speed up PROPFIND searches
     * without the need for tagging URIs clean or dirty.
     */
    private boolean flushOnCommandDone = true;
    
    
    // ------------------------------------------------------------- Properties
    
    
    /**
     * Returns the strategy.
     */
    public String getStrategy() {
         return strategy;
    }
    
    
    /**
     * set the strategy
     */
    public void setStrategy(String strategy) {
        this.strategy = strategy;
    }
    
    /**
     * Returns the searchDepth.
     */
    public int getSearchDepth() {
         return searchDepth;
    }
    
    
    /**
     * set the searchDepth
     */
    public void setSearchDepth(int searchDepth) {
        this.searchDepth = searchDepth;
    }
    
    
    /**
     * Returns the searchDepth.
     */
    public boolean isFlushOnCommandDone() {
         return flushOnCommandDone;
    }
    
    
    /**
     * set the flushOnCommandDone
     */
    public void setFlushOnCommandDone(boolean flushOnCommandDone) {
        this.flushOnCommandDone = flushOnCommandDone;
    }
    
    
}
