/*
 * $Header: /home/cvs/jakarta-slide/src/share/org/apache/slide/security/NodePermission.java,v 1.8 2001/07/13 13:55:33 dirkv Exp $
 * $Revision: 1.8 $
 * $Date: 2001/07/13 13:55:33 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 

package org.apache.slide.security;

import java.util.Enumeration;
import java.io.Serializable;
import org.apache.slide.common.*;
import org.apache.slide.structure.*;
import org.apache.slide.content.NodeRevisionNumber;
import org.apache.slide.util.Messages;

/**
 * Permission object.
 * 
 * @author <a href="mailto:remm@apache.org">Remy Maucherat</a>
 * @version $Revision: 1.8 $
 */
public final class NodePermission implements Serializable, Cloneable {
    
    
    // ----------------------------------------------------------- Constructors
    
    
    /**
     * Constructor 1.
     * 
     * @param objectUri Object Uri
     * @param subjectUri Subject Uri
     * @param actionUri Action Uri
     */
    public NodePermission(String objectUri, String subjectUri, 
                          String actionUri) {
        if (objectUri != null) {
            this.objectUri = objectUri;
	}
        if (subjectUri != null) {
            this.subjectUri = subjectUri;
        }
        if (actionUri != null) {
            this.actionUri = actionUri;
        }
        inheritable = true;
        negative = false;
    }
    
    
    /**
     * Constructor 2.
     * 
     * @param objectUri Object Uri
     * @param subjectUri Subject Uri
     * @param actionUri Action Uri
     * @param inheritable Inheritance flag
     */
    public NodePermission(String objectUri, String subjectUri, 
                          String actionUri, boolean inheritable) {
        this(objectUri, subjectUri, actionUri);
        this.inheritable = inheritable;
    }
    
    
    /**
     * Constructor 3.
     * 
     * @param objectUri Object Uri
     * @param subjectUri Subject Uri
     * @param actionUri Action Uri
     * @param inheritable Inheritance flag
     * @param negative Negative flag
     */
    public NodePermission(String objectUri, String subjectUri, 
                          String actionUri, boolean inheritable,
                          boolean negative) {
        this(objectUri, subjectUri, actionUri);
        this.inheritable = inheritable;
        this.negative = negative;
    }
    
    
    /**
     * Constructor 4.
     * 
     * @param object Object
     * @param subject Subject
     * @param action Action
     */
    public NodePermission(ObjectNode object, SubjectNode subject, 
                          ActionNode action) {
        this(object.getUri(), subject.getUri(), action.getUri());
    }
    
    
    /**
     * Constructor 5.
     * 
     * @param object Object
     * @param subject Subject
     * @param action Action
     * @param inheritable Inheritance flag
     */
    public NodePermission(ObjectNode object, SubjectNode subject, 
                          ActionNode action, boolean inheritable) {
        this(object, subject, action);
        this.inheritable = inheritable;
    }
    
    
    /**
     * Constructor 6.
     * 
     * @param object Object
     * @param subject Subject
     * @param action Action
     * @param inheritable Inheritance flag
     * @param negative Negative flag
     */
    public NodePermission(ObjectNode object, SubjectNode subject, 
                          ActionNode action, boolean inheritable,
                          boolean negative) {
        this(object, subject, action);
        this.inheritable = inheritable;
        this.negative = negative;
    }
    
    
    /**
     * Constructor 7.
     * 
     * @param object Object
     * @param revisionNumber Revision number
     * @param subject Subject
     * @param action Action
     * @param inheritable Inheritance flag
     * @param negative Negative flag
     */
    public NodePermission(ObjectNode object, NodeRevisionNumber revisionNumber,
                          SubjectNode subject, ActionNode action, 
                          boolean inheritable, boolean negative) {
        this(object, subject, action);
        this.inheritable = inheritable;
        if (revisionNumber != null)
            this.inheritable = false;
        this.negative = negative;
        this.revisionNumber = revisionNumber;
    }
    
    
    /**
     * Constructor 8.
     * 
     * @param objectUri Object Uri
     * @param subjectUri Subject Uri
     * @param actionUri Action Uri
     * @param inheritable Inheritance flag
     * @param negative Negative flag
     */
    public NodePermission(String objectUri, String revisionNumber,
                          String subjectUri, String actionUri, 
                          boolean inheritable, boolean negative) {
        this(objectUri, subjectUri, actionUri);
        this.inheritable = inheritable;
        if (revisionNumber != null) {
            this.revisionNumber = new NodeRevisionNumber(revisionNumber);
            this.inheritable = false;
        }
        this.negative = negative;
    }
    
    
    // ----------------------------------------------------- Instance Variables
    
    
    /**
     * Object Uri.
     */
    protected String objectUri;
    
    
    /**
     * Revision number.
     */
    protected NodeRevisionNumber revisionNumber;
    
    
    /**
     * Subject Uri.
     */
    protected String subjectUri;
    
    
    /**
     * Action Uri.
     */
    protected String actionUri;
    
    
    /**
     * Inheritance flag. True if permission is inheritable.
     */
    protected boolean inheritable;
    
    
    /**
     * Negative permission flag.
     */
    protected boolean negative;
    
    
    // ------------------------------------------------------------- Properties
    
    
    /**
     * Inheritance flag mutator.
     * 
     * @param inheritable New flag value
     */
    public void setInheritable(boolean inheritable) {
        this.inheritable = inheritable;
    }
    
    
    /**
     * Inheritance flag accessor.
     * 
     * @return boolean True if permission is inheritable
     */
    public boolean isInheritable() {
        return inheritable;
    }
    
    
    /**
     * Negative flag mutator.
     * 
     * @param negative New negative value
     */
    public void setNegative(boolean negative) {
        this.negative = negative;
    }
    
    
    /**
     * Negative flag accessor.
     * 
     * @return boolean True if permission is negative
     */
    public boolean isNegative() {
        return negative;
    }
    
    
    /**
     * Object Uri accessor.
     * 
     * @return String Object Uri
     */
    public String getObjectUri() {
        return objectUri;
    }
    
    
    /**
     * Object Uri mutator.
     * 
     * @param object New object
     */
    void setObject(ObjectNode object) {
        if (object != null) {
            this.objectUri = object.getUri();
        }
    }
    
    
    /**
     * Object Uri mutator
     * 
     * @param objectUri New object Uri
     */
    void setObject(String objectUri) {
        this.objectUri = objectUri;
    }
    
    
    /**
     * Subject Uri mutator.
     * 
     * @return String Subject Uri
     */
    public String getSubjectUri() {
        return subjectUri;
    }
    
    
    /**
     * Subject Uri mutator.
     * 
     * @param subject New subject
     */
    void setSubject(SubjectNode subject) {
        if (subject != null) {
            this.subjectUri = subject.getUri();
        }
    }
    
    
    /**
     * Subject Uri mutator.
     * 
     * @param subjectUri New subject Uri
     */
    void setSubject(String subjectUri) {
        this.subjectUri = subjectUri;
    }
    
    
    /**
     * Revision number accessor.
     * 
     * @return Revision number to which this permission applies
     */
    public NodeRevisionNumber getRevisionNumber() {
        return revisionNumber;
    }
    
    
    /**
     * Revision number mutator.
     * 
     * @param revisionNumber Revision number
     */
    void setRevisionNumber(NodeRevisionNumber revisionNumber) {
        this.revisionNumber = revisionNumber;
    }
    
    
    /**
     * Action Uri accessor.
     * 
     * @return String Action Uri
     */
    public String getActionUri() {
        return actionUri;
    }
    
    
    /**
     * Action Uri mutator.
     * 
     * @param action New action
     */
    void setAction(ActionNode action) {
        if (action != null) {
            this.actionUri = action.getUri();
        }
    }
    
    
    /**
     * Action Uri mutator.
     * 
     * @param actionUri New action Uri
     */
    void setAction(String actionUri) {
        this.actionUri = actionUri;
    }
    
    
    // --------------------------------------------------------- Object Methods
    
    
    /**
     * Equals.
     * 
     * @param obj Object to test
     * @return boolean True if the two object are equal :
     * <li>obj is of type SlidePermission and is not null</li>
     * <li>All three Uris are equal</li>
     */
    public boolean equals(Object obj) {
        boolean result = false;
        if ((obj != null) && (obj instanceof NodePermission)) {
            NodePermission permission = (NodePermission) obj;
            result = (this.objectUri.equals(permission.getObjectUri())) 
                && (this.subjectUri.equals(permission.getSubjectUri()))
                && (this.actionUri.equals(permission.getActionUri()));
        }
        return result;
    }
    
    
    /**
     * Hash Code.
     * 
     * @return int Hash code value
     */
    public int hashCode() {
        return toString().hashCode();
    }
    
    
    /**
     * String representation of the permission.
     * <p/>
     * Format : ObjectUri-SubjectUri-ActionUri-InheritanceFlag
     * 
     * @return String String representation
     */
    public String toString() {
        return objectUri + "-" + subjectUri + "-" + actionUri + "-" 
            + inheritable;
    }
    
    
    /**
     * Clone.
     * 
     * @return Object clone
     */
    public NodePermission cloneObject() {
        NodePermission result = null;
        try {
            result = (NodePermission) super.clone();
        } catch(CloneNotSupportedException e) {
        }
        return result;
    }

    
    /**
     * Validate.
     * 
     * @param expectedUri Uri
     */
    public void validate(String expectedUri) {
        
        if (objectUri == null)
            throw new ObjectValidationFailedException
                (expectedUri, Messages.message
                 (NodePermission.class.getName() + ".nullObjectUri"));
        
        if (!objectUri.equals(expectedUri)) 
		{
			// extra test to make "/files/x" == "/files/x/"
			String tmpObjectUri=objectUri;
			if (tmpObjectUri.endsWith("/"))
				tmpObjectUri=tmpObjectUri.substring(0,tmpObjectUri.length()-1);
			String tmpExpectedUri=expectedUri;
			if (tmpExpectedUri.endsWith("/"))
				tmpExpectedUri=tmpExpectedUri.substring(0,tmpExpectedUri.length()-1);

           	if (!tmpObjectUri.equals(tmpExpectedUri)) 
				throw new ObjectValidationFailedException
				    (expectedUri, Messages.message
				     (NodePermission.class.getName() + ".incorrectObjectUri"));
		}
        
        if (subjectUri == null)
            throw new ObjectValidationFailedException
                (expectedUri, Messages.message
                 (NodePermission.class.getName() + ".nullSubjectUri"));
        
        if (actionUri == null)
            throw new ObjectValidationFailedException
                (expectedUri, Messages.message
                 (NodePermission.class.getName() + ".nullActionUri"));
        
    }
    
    
}
